package DeleteCategoryRebuilder::CMS;

use strict;
use File::Basename;

use Data::Dumper;

sub rebuild_delete_category_archive {
    my ($eh, $app, $obj, $orig_obj) = @_;

    my ($terms, $args);
    $terms->{entry_id} = $obj->id;
    $args->{sort} = 'rev_number';
    $args->{direction} = 'descend';
    $args->{limit} = 5;
    my $rev_model = $app->model('entry:revision');
    my @revs = $rev_model->load($terms, $args);
    my $flag = 0;
    my $data;
    for my $revision (@revs) {
        $data = unpack_entry( $obj, $revision );
        if ($data->{status} == 2) {
            $flag = 1;
            last;
        }
    }
    return if !$flag; # no object
    return if !$data;

    my $cats = $data->{categories};
    my @cat_ids;
    for my $cat (@$cats) {
        push @cat_ids, $cat->id;
    }

    my @current_categories = split(',', $app->param('category_ids'));
    # return if !@current_categories; # must not return!

    my @rebuid_cat_ids;
    for my $old_cat_id (@cat_ids) {
        $flag = 0;
        for my $new_cat_id (@current_categories) {
            if ($new_cat_id == $old_cat_id) {
                $flag = 1;
            }
        }
        push @rebuid_cat_ids, $old_cat_id if !$flag;
    }
    return if !@rebuid_cat_ids;

    # add parent categories
    my @rebuild_cats = MT::Category->load({ id => \@rebuid_cat_ids, blog_id => $obj->blog_id });

    my $plugin = MT->component('DeleteCategoryRebuilder');
    my $rebuild_parent = $plugin->get_config_value('rebuild_parent', 'blog:' . $app->blog->id);
    if ( $rebuild_parent ) {
        for my $cat (@rebuild_cats) {
            if ($cat->parent) {
                my $parent_id = $cat->parent;
                while ($parent_id) {
                    my $parent = MT::Category->load({ id => $parent_id, blog_id => $obj->blog_id });
                    push @rebuild_cats, $parent;
                    $parent_id = $parent->parent ? $parent->parent : '';
                }
            }
        }
    }
    my $blog = $app->blog;
    my %param;
    my $at = $param{PreferredArchiveOnly}
        ? $blog->archive_type_preferred
        : $blog->archive_type;

    my @cat_archive_types;
    my @archive_types = split(',', $at);
    for my $archive_type (@archive_types) {
        push @cat_archive_types, $archive_type if $archive_type =~ /Category/;
    }
    return if !@cat_archive_types;

    for my $at (@cat_archive_types) {
        for my $cat (@rebuild_cats) {
            _rebuild_entry_archive_type(
                Entry       => $obj,
                Blog        => $blog,
                ArchiveType => $at,
                Category    => $cat,
                NoStatic    => $param{NoStatic},

               # Force       => ($param{Force} ? 1 : 0),
                $param{TemplateMap}
                ? ( TemplateMap => $param{TemplateMap} )
                : (),
            ) or return;
        }
    }
    return 1;
}

sub unpack_entry {
    my ( $obj, $rev ) = @_;
    my $datasource = $obj->datasource;
    my $serialized_obj = $rev->$datasource;
    require MT::Serialize;
    my $packed_obj = MT::Serialize->unserialize($serialized_obj);

    my ( %data, @cats );
    $data{status} = $$packed_obj->{status};

    if ( my $rev_cats = delete $$packed_obj->{__rev_cats} ) {
        if (@$rev_cats) {
            my $cats = MT::Category->lookup_multi(
                [ map { $_->[0] } @$rev_cats ] );
            @cats = sort { $a->label cmp $b->label } grep {defined} @$cats;
        }
        $data{categories} = \@cats;
    }
    return \%data;
}

sub _rebuild_entry_archive_type {
    my %param = @_;
    my $mt = MT->instance;
    my $at = $param{ArchiveType}
        or return $mt->error(
        MT->translate( "Parameter '[_1]' is required", 'ArchiveType' ) );
    return 1 if $at eq 'None';

    my $entry
        = (    $param{ArchiveType} ne 'Category'
            && $param{ArchiveType} ne 'Author'
            && !exists $param{Start}
            && !exists $param{End} )
        ? (
        $param{Entry}
            or return $mt->error(
            MT->translate( "Parameter '[_1]' is required", 'Entry' )
            )
        )
        : undef;

    my $blog;
    unless ( $blog = $param{Blog} ) {
        my $blog_id = $entry->blog_id;
        $blog = MT::Blog->load($blog_id)
            or return $mt->error(
            MT->translate(
                "Load of blog '[_1]' failed: [_2]", $blog_id,
                MT::Blog->errstr
            )
            );
    }

    require MT::TemplateMap;
    my @map;
    my $cached_maps = MT->instance->request('__cached_maps')
        || MT->instance->request( '__cached_maps', {} );
    if ( my $maps = $cached_maps->{ $at . $blog->id } ) {
        @map = @$maps;
    }
    else {
       @map = MT::TemplateMap->load(
           {   archive_type => $at,
               blog_id      => $blog->id,
                $param{TemplateID}
                ? ( template_id => $param{TemplateID} )
                : ()
           }
       );
        $cached_maps->{ $at . $blog->id } = \@map;
    }
    return 1 unless @map;

    my @map_build;
    my $done = MT->instance->request( '__published:' . $blog->id )
        || MT->instance->request( '__published:' . $blog->id, {} );
    for my $map (@map) {
        my $file
            = exists $param{File}
            ? $param{File}
            : MT::Util::archive_file_for( $entry, $blog, $at, $param{Category},
            $map, undef, $param{Author} );
        if ( $file eq '' ) {

            # np
        }
        elsif ( !defined($file) ) {
            return $mt->error( MT->translate( $blog->errstr() ) );
        }
        else {
            push @map_build, $map unless $done->{$file};
            $map->{__saved_output_file} = $file;
        }
    }
    return 1 unless @map_build;
    @map = @map_build;

    $at ||= "";

    my $pub = $mt->publisher;
    my $archiver = $pub->archiver($at);
    return unless $archiver;

    my $arch_root = $blog->archive_path;
    return $mt->error(
        MT->translate("You did not set your blog publishing path") )
        unless $arch_root;

    my ( $start, $end );
    if ( exists $param{Start} && exists $param{End} ) {
        $start = $param{Start};
        $end   = $param{End};
    }
    else {
        if ( $archiver->date_based() && $archiver->can('date_range') ) {
            ( $start, $end ) = $archiver->date_range( $entry->authored_on );
        }
    }

    require MT::Template;
    require MT::Template::Context;
    require MT::PublishOption;

    my $force = $param{Force};
    for my $map (@map) {
        # my $dynamic = $map->build_type == MT::PublishOption::DYNAMIC() ? 1: 0;
        next unless $map->build_type;    # ignore disabled template maps
        next if $map->build_type == MT::PublishOption::MANUALLY() && !$force;

        my $ctx = MT::Template::Context->new;
        $ctx->{current_archive_type} = $at;
        $ctx->{archive_type}         = $at;
        $pub->rebuild_file(
            $blog, $arch_root, $map, $at, $ctx, \my %cond,
            !$param{NoStatic},
            Category  => $param{Category},
            Entry     => $entry,
            Author    => $param{Author},
            StartDate => $start,
            EndDate   => $end,
            Force     => $param{Force} ? 1 : 0,
        ) or return;
        $done->{ $map->{__saved_output_file} }++;
    }
    1;
}

1;
